/* -*- Mode: C; c-file-style: "bsd" -*- */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include "db.h"
#include "types.h"

extern int verbose_flag;

/* dumb, implementations of a database function - linear lookup */

const char* db_filename = "hashcash.db";
char today[ 6 ];
word32 today_num;

void db_create_if_nec( void );
void db_purge_if_nec( void );

void db_open( const char* database_filename )
{
    today_num = ( time( 0 ) / 3600 ) / 24;
    sprintf( today, "%05d", today_num );
    db_filename = database_filename;
}

void db_close() { }

void db_add( const char* collision, int expiry )
{
    FILE* db;

    db_create_if_nec();
    db_purge_if_nec();

    db = fopen( db_filename, "a" );
    if ( db == 0 )
    {
	perror( db_filename );
	exit( 1 );
    }

    if ( verbose_flag ) { printf( "adding: %d %s\n", expiry, collision ); }
    fseek( db, 0, SEEK_END );
    fprintf( db, "%d ", expiry );
    fputs( collision, db );
    fputc( '\n', db );
    fclose( db );
}

int db_in( const char* find )
{
    char entry[ 100 ];
    char* collision;
    char* expiry;
    int expiry_num;
    char day[ 6 ];
    word32 day_num; 
    int len;
    FILE* db;

    db_create_if_nec();
    db_purge_if_nec();

    len = strlen( find );

    db = fopen( db_filename, "r" );

    if ( db == 0 )
    {
	perror( db_filename );
	exit( 1 );
    }

    fgets( entry, 100, db );	/* junk date line */

    while ( !feof( db ) )
    {
	fgets( entry, 100, db );
	if ( !feof( db ) )
	{
	    expiry = strtok( entry, " " );
	    if ( expiry == 0 ) 
	    {
		fclose( db );
		return 0;
	    }
	    collision = strtok( 0, " " );
	    if ( strncmp( find, collision, len ) == 0 )
	    {
		fclose( db );
		return 1;
	    }
	}
    }
    fclose( db );
    return 0;
}

void db_purge( void )
{
    FILE* db = fopen( db_filename, "r" );
    char purged_filename[ 256 ];
    FILE* purged;
    int status;
    char entry[ 100 ];
    char day[ 6 ];
    word32 day_num;
    char* expiry;
    char* collision;
    word32 expiry_num;

    if ( verbose_flag ) { printf( "purging database...\n" ); }

    if ( db == 0 )
    {
	perror( db_filename );
	exit( 1 );
    }

    strncpy( purged_filename, db_filename, 256 );
    strcat( purged_filename, "1" );

    purged = fopen( purged_filename, "w" );

    if ( purged == 0 )
    {
	perror( purged_filename );
	exit( 1 );
    }

    if ( verbose_flag ) { printf( "tmpfile: %s\n", purged_filename ); }
    fputs( today, purged );
    fputc( '\n', purged );

    fgets( entry, 100, db );	/* junk date line */
    while ( ! feof( db ) )
    {
	fgets( entry, 100, db );
	expiry = strtok( entry, " " );
	if ( expiry == 0 )
	{
	    continue;
	}
	collision = strtok( 0, " " );
	if ( collision == 0 )
	{
	    continue;
	}
	expiry_num = atoi( expiry );
	strncpy( day, collision, 5 );
	day[ 5 ] ='\0';
	day_num = atoi( day );
	if ( expiry_num == 0 || expiry_num && day_num + expiry_num >= today_num )
	{
	    fputs( expiry, purged );
	    fputc( ' ', purged );
	    fputs( collision, purged );
	}
	else
	{
	    if ( verbose_flag )
	    {
		printf( "discarded: %d %s", expiry_num, collision );
	    }
	}
    }
    fflush( purged );
    fclose( purged );
    fclose( db );
    unlink( db_filename );
    status = rename( purged_filename, db_filename );
    if ( status < 0 )
    {
	perror( "rename" );
	exit( 1 );
    }
    if ( verbose_flag ) { printf( "done\n" ); }
}

void db_set_filename( const char* filename )
{
    db_filename = filename;
}

void db_create_if_nec( void )
{
    FILE* db = fopen( db_filename, "r" );
    if ( db == 0 )
    {
	if ( errno == ENOENT )
	{
	    db = fopen( db_filename, "w" );
	    if ( db == 0 )
	    {
		perror( db_filename );
		exit( 1 );
	    }
	    fprintf( db, "%s\n", today );
	    fclose( db );
	}
    }
}

void db_purge_if_nec( void )
{
    FILE* db;
    char day[ 6 ];

    db = fopen( db_filename, "r" );
    fgets( day, 6, db );
    day[ 5 ] ='\0';
    fclose( db );

    if ( strncmp( day, today, 5 ) != 0 )
    {
	db_purge();
    }
}

